//	Camera.h
//
//	Project : Camera Library for DT-5100
//	Author  : IRIE Tomonori
//	Date    : 2003-02-19
//	Version : 1.00.00
//	Copyright (C) 2003 CASIO COMPUTER CO.,LTD.
//	CASIO CONFIDENTIAL
//
//	History
//	2003-02-19 (1.00.00 ) : First Released

#ifndef __CAMERA_H__
#define __CAMERA_H__

// Return Codes
// General
#define CAM_SUCCESS				0
#define CAM_NOTOPENED			-1
#define CAM_INVALID_PARAM		-2
#define CAM_INTERNAL_ERROR		-3
// CAM_Open
#define CAM_OPENERROR			-11
#define CAM_NOT_ENOUGH_MEMORY	-12
// CAM_StartPreview
#define CAM_SHOW_ERROR			-21
// CAM_StopPreview
#define CAM_NOT_STARTED			-31
// CAM_SingleCapture
#define CAM_FILE_ERROR			-41

// Capture Size
#define CAM_VGA					1
#define CAM_4PER9VGA			2
#define CAM_QVGA				3
#define CAM_1PER9VGA			4
#define CAM_QQVGA				5

// Image Format
#define CAM_BMP					0
#define CAM_JPEG				1

// Image Settings
#define CAM_JPEG_LOW			50
#define CAM_JPEG_NORMAL			75
#define CAM_JPEG_HIGH			85
#define CAM_JPEG_HIGHEST		95
#define CAM_JPEG_BASELINE		1
#define CAM_JPEG_PROGRESSIVE	2

// Frame Rates
#define CAM_5FPS				5
#define CAM_15FPS				15
#define CAM_10FPS				10

// Light Mode
#define CAM_LIGHT_AUTO			0
#define CAM_OUTDOOR				1
#define CAM_FLUORESCENT			2
#define CAM_INCANDESCE			3

// Digital Zoom Magnification
#define CAM_ZOOM_NONE			1
#define CAM_ZOOM_15				2
#define CAM_ZOOM_20				3
#define CAM_ZOOM_30				4

// Sharpness Correction
#define CAM_SHARPNESS_NONE		0
#define CAM_SHARPNESS_WEAK		1
#define CAM_SHARPNESS_MIDDLE	2
#define CAM_SHARPNESS_STRONG	3


// Structures
// Image Option Table
typedef struct _cam_image_settings
{
	INT nQuality;
	INT nJpegOpt;
} CAM_IMAGE_SET;


#ifdef __cplusplus
extern "C" {
#endif

// Functions
INT CAMOpen( VOID);									// Loads the Camera Driver
INT CAMClose( VOID);								// Unloads the Camera Driver
INT CAMStartPreview( HWND, INT, INT, INT);			// Starts the preview-screen
INT CAMStopPreview( VOID);							// Stops the preview-screen
INT CAMSingleCapture( const TCHAR *, INT, INT);		// Captures a single image
INT CAMGetImageSettings( CAM_IMAGE_SET *);			// Gets the image settings
INT CAMSetImageSettings( const CAM_IMAGE_SET *);	// Sets the image settings
INT CAMContinuousCapture( const TCHAR *, INT, INT, INT);
													// Captures continuous images
INT CAMGetLightMode( VOID);							// Gets the white-balance mode
INT CAMSetLightMode( INT);							// Sets the white-balance mode
INT CAMGetDigitalZoom( VOID);						// Gets the zoom magnification
INT CAMSetDigitalZoom( INT);						// Sets the zoom magnification
INT CAMGetSensitivity( VOID);						// Gets the sensitivity
INT CAMSetSensitivity( INT);						// Sets the sensitivity
INT CAMRedrawPreview( VOID);						// Redraws the preview-screen

#ifdef __cplusplus
}
#endif

#endif
