//	Sample.cpp
//
//	Project : Sample Program for DT-5100 Camera
//	Date    : 2003-01-17
//	Version : 1.00
//	
//	Copyright (C) 2003 CASIO COMPUTER CO.,LTD.
//	CASIO CONFIDENTIAL

#include "stdafx.h"
#include "Sample.h"
#include <commctrl.h>
#include "Camera.h"

#define MAX_LOADSTRING	100
#define KEY_L			'L'

// Global Variables:
HINSTANCE	hInst;				// The current instance
HWND		hwndCB;				// The command bar handle
BOOL		g_bPreviewFlag;		// If previewing then TRUE, else FALSE
const char g_ProgramVersion[] = "Camera sample program version 1.00";

// Forward declarations of functions included in this code module:
ATOM MyRegisterClass		(HINSTANCE, LPTSTR);
BOOL InitInstance			(HINSTANCE, int);
LRESULT CALLBACK	WndProc	(HWND, UINT, WPARAM, LPARAM);
BOOL GetFileName			( TCHAR *);
void PathRemoveFileSpec		( TCHAR *);

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;

	if ( !InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	g_bPreviewFlag = FALSE;
	if ( CAMOpen() != CAM_SUCCESS)
	{
		MessageBox( NULL, TEXT("CAMOpen Error!"),
			TEXT("Camera Sample"), MB_ICONSTOP);
	}

	while ( GetMessage( &msg, NULL, 0, 0)) 
	{
		if ( !TranslateAccelerator( msg.hwnd, 0, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return msg.wParam;
}

ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS	wc;

    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= 0;
    wc.hCursor			= 0;
    wc.hbrBackground    = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND	hWnd;
	TCHAR	szTitle[MAX_LOADSTRING];			// The title bar text
	TCHAR	szWindowClass[MAX_LOADSTRING];		// The window class name

	hInst = hInstance;		// Store instance handle in our global variable
	// Initialize global strings
	LoadString(hInstance, IDC_SAMPLE, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance, szWindowClass);

	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		NULL, NULL, hInstance, NULL);

	if (!hWnd)
	{	
		return FALSE;
	}

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);
	if (hwndCB)
		CommandBar_Show(hwndCB, TRUE);

	return TRUE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;

	MSG Msg;

	switch (message) 
	{
		case WM_HOTKEY:
			switch( wParam)
			{
			case KEY_L:											// L Key
				// Camera Shutter
				PostMessage( hWnd, WM_COMMAND, IDC_BTNSHUTTER, 0);
				break;
			}
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDC_GETSENS:
					// Shows the current sensitivity value
					SetDlgItemInt(hWnd,IDC_EDITEC, CAMGetSensitivity(), FALSE);
					break;
				case IDC_BTNSTART:
					// Starts previewing
					if(CAMStartPreview(hWnd, 12, 131, CAM_1PER9VGA)==CAM_SUCCESS)
					{
						EnableWindow(GetDlgItem(hWnd,IDC_BTNSTART)  ,FALSE);
						EnableWindow(GetDlgItem(hWnd,IDC_BTNSTOP)   ,TRUE);
						EnableWindow(GetDlgItem(hWnd,IDC_BTNSHUTTER),TRUE);
						g_bPreviewFlag = TRUE;
					}
					else
					{
						MessageBox(hWnd, TEXT("CAMStartPreview Error!"),
							TEXT("Camera Sample"), MB_ICONSTOP);
					}
					break;
				case IDC_BTNSTOP:
					// Stops the previewing
					if(CAMStopPreview()==CAM_SUCCESS)
					{
						EnableWindow(GetDlgItem(hWnd,IDC_BTNSTART)  ,TRUE);
						EnableWindow(GetDlgItem(hWnd,IDC_BTNSTOP)   ,FALSE);
						EnableWindow(GetDlgItem(hWnd,IDC_BTNSHUTTER),FALSE);
						g_bPreviewFlag = FALSE;
					}else
					{
						MessageBox(hWnd, TEXT("CAMStopPreview Error!"),
							TEXT("Camera Sample"), MB_ICONSTOP);
					}
					break;
				case IDC_BTNUP:
				case IDC_BTNDOWN:
				{
					// Sets the sensitivity
					BOOL bRet;
					int nData = GetDlgItemInt(hWnd, IDC_EDITEC, &bRet, FALSE);

					if(LOWORD(wParam) == IDC_BTNUP) nData++;
					else                            nData--;

					if(nData<0 || nData>24)
					{
						MessageBox(hWnd, TEXT("Sensitivity is from 0 to 24."),
							TEXT("Camera Sample"), MB_ICONSTOP);
						break;
					}
					SetDlgItemInt(hWnd,IDC_EDITEC, nData, FALSE);
					CAMSetSensitivity(nData);
					break;
				}
				case IDC_BTNSHUTTER:
				{
					// Captures a VGA size picture
					TCHAR lBuf[256];

					SetCursor(LoadCursor(NULL, IDC_WAIT));
					EnableWindow(GetDlgItem(hWnd,IDC_BTNSHUTTER),FALSE);

					GetDlgItemText(hWnd, IDC_EDITFILENAME, lBuf, sizeof(lBuf)-2);
					if( !GetFileName( lBuf))
					{
						MessageBox(hWnd, TEXT("Please specify a file name."),
							TEXT("Camera Sample"), MB_ICONSTOP);
						SetFocus(GetDlgItem(hWnd,IDC_EDITFILENAME));
						break;
					}

					if(CAMSingleCapture(lBuf, CAM_JPEG, CAM_VGA)!=CAM_SUCCESS)
					{
						MessageBox(hWnd, TEXT("CAMSingleCapture Error!"),
							TEXT("Camera Sample"), MB_ICONSTOP);
						break;
					}

					EnableWindow(GetDlgItem(hWnd,IDC_BTNSHUTTER),TRUE);
					SetCursor( LoadCursor( NULL, IDC_ARROW));

					while( PeekMessage(&Msg , hWnd, WM_KEYFIRST, 786, PM_REMOVE))
					{
						Sleep( 1);
					}
					break;
				}
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_CREATE:
			hwndCB = CommandBar_Create(hInst, hWnd, 0);			
			CommandBar_AddAdornments(hwndCB, 0, 0);
    
			// File Name
			CreateWindow( TEXT( "static"), TEXT("File Name:"),
				WS_CHILD | WS_VISIBLE,  5, 27, 60, 20, hWnd, 0, hInst, NULL);
			CreateWindow(TEXT("EDIT"),TEXT("Sample1.jpg"),WS_CHILD | WS_VISIBLE |	WS_BORDER,
						67,25,163,20,hWnd,(HMENU)IDC_EDITFILENAME,hInst,NULL);

			// [Preview Start]Button
			CreateWindow( TEXT("button"), TEXT("Preview Start"),
				WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 5,  47, 110, 20, hWnd, ( HMENU)IDC_BTNSTART, hInst, NULL);

			// [Preview Stop]Button
			CreateWindow( TEXT("button"), TEXT("Preview Stop"),
				WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 120,  47, 110, 20, hWnd, ( HMENU)IDC_BTNSTOP, hInst, NULL);

			// [Shutter!]Button
			CreateWindow( TEXT("button"), TEXT("Shutter!"),
				WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 5,  70, 225, 20, hWnd, ( HMENU)IDC_BTNSHUTTER, hInst, NULL);

			// Sensitivity (Exposure Correction)
			CreateWindow(TEXT("EDIT"),NULL,WS_CHILD | WS_VISIBLE | WS_BORDER | WS_DISABLED ,
						36,108,60,20,hWnd,(HMENU)IDC_EDITEC,hInst,NULL);
			CreateWindow( TEXT("button"), TEXT("Up"),
				WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 101,  108, 62, 20, hWnd, ( HMENU)IDC_BTNUP, hInst, NULL);
			CreateWindow( TEXT("button"), TEXT("Down"),
				WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON,
				 168,  108, 62, 20, hWnd, ( HMENU)IDC_BTNDOWN, hInst, NULL);
			CreateWindow( TEXT( "static"), TEXT("Exposure Correction:"),
				WS_CHILD | WS_VISIBLE,  5, 91, 160, 20, hWnd, 0, hInst, NULL);

			EnableWindow(GetDlgItem(hWnd,IDC_BTNSTOP), FALSE);
			EnableWindow(GetDlgItem(hWnd,IDC_BTNSHUTTER), FALSE);
			
			//Sensitivity is assigned 
			PostMessage(hWnd, WM_COMMAND, IDC_GETSENS, 0);

			// Set L-Key as a HotKey
			if ( RegisterHotKey( hWnd, KEY_L, 0, 0x0087) == FALSE)
			{
				MessageBox( hWnd, TEXT("Cannot use L-Key!"), NULL, MB_ICONSTOP);
			}

			break;

		case WM_PAINT:
		{
			RECT rt;
			hdc = BeginPaint(hWnd, &ps);
			GetClientRect(hWnd, &rt);

			CAMRedrawPreview();
			EndPaint(hWnd, &ps);
			break;
		}
		case WM_CTLCOLORSTATIC:
			return (long)GetStockObject(WHITE_BRUSH);
		case WM_DESTROY:
			if ( CAMClose() != CAM_SUCCESS)
			{
				MessageBox( NULL, TEXT("CAMClose Error!"),
					TEXT("Camera Sample"), MB_ICONSTOP);
			}

			UnregisterHotKey( hWnd, KEY_L);

			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Sets the output file path
BOOL GetFileName( TCHAR *fileName)
{
	TCHAR lBuf[256];
	DWORD fileNameSize;
	PROCESS_INFORMATION pi;

	if(*fileName==0) return FALSE;

	memset( &pi, 0, sizeof(PROCESS_INFORMATION) );
	lBuf[0]=0;

	fileNameSize = GetModuleFileName(NULL, lBuf, sizeof(lBuf)-2);
	if (fileNameSize){
		PathRemoveFileSpec(lBuf);
		wcscat(lBuf, fileName);
		wcscpy(fileName, lBuf);
	}

	return TRUE;
}

// Catches up the path excluding the file name
void PathRemoveFileSpec( TCHAR *fileName )
{
	TCHAR *ptr = fileName + _tcslen(fileName);
	while(1){
		ptr--;
		if ((*ptr == TEXT('\\')) || (*ptr == TEXT('/')))
		{
			*(++ptr) = TEXT('\0');
			break;
		}
		else if ((*ptr == TEXT('\0')) || (ptr == fileName))
		{
			break;
		}
	}
	return;
}
